#! /usr/bin/python
# Copy or move Artistic Style backup files to a backup directory.
# NOTE: This script uses "shutil.copy2" which will keep the
#     last access time and last modification time of the original file.

import os
import shutil
import sys

# global variables ------------------------------------------------------------

# CHANGE THE FOLLOWING 4 VARIABLES
# g_indir is the input top level directory containing the .orig files
# g_outdir is the output top level directory containing the moved files
# g_fileext is the backup file extension
# g_copyfiles is COPY or MOVE the files

g_indir = ".."
g_outdir = "../BACKUP_LIBRARY"
g_fileext = ".orig"
# USE ONE OF THE FOLLOWING TO COPY OR MOVE THE FILES
#g_copyfiles = "COPY"
g_copyfiles = "MOVE"

# -----------------------------------------------------------------------------

def process_directories():
	"""Main processing function"""
	"""Walk thru the top-level directory tree."""
	global g_indir, g_outdir
	# initialization
	g_indir = os.path.normpath(os.path.expanduser(g_indir))
	g_outdir = os.path.normpath(os.path.expanduser(g_outdir))
	display_global_variables()
	if not validate_global_variables(): return
	processed = 0		# number of files processed
	print g_indir
	# walk thru the directory tree
	for dirpath, dirnames, filenames in os.walk(g_indir):
		if dirpath != g_indir:
			print "directory: " + dirpath[len(g_indir)+1:]
		remove_hidden_directories(dirnames)
		processed += process_backup_files(dirpath, filenames)
	print "{0} \"{1}\" files processed".format(processed, g_fileext)

# -----------------------------------------------------------------------------

def display_global_variables():
	"""Display the global variables."""
	print
	print g_copyfiles + " Artistic Style backup files"
	print "FROM " + g_indir
	print "TO   " + g_outdir
	print

# -----------------------------------------------------------------------------

def move_or_copy_file(filepath, outpath, file):
	"""Move or copy a file to the backup directory."""
	outdir = outpath[:-len(file)-1]
	# create a directory
	if not os.path.isdir(outdir):
		os.makedirs(outdir)
	# copy to backup
	shutil.copy2(filepath, outpath)
	# remove from filepath
	if g_copyfiles == "MOVE":
		os.remove(filepath)
		
# -----------------------------------------------------------------------------

def process_backup_files(dirpath, filenames):
	"""Process the backup files in a directory."""
	"""Return the number of files processed."""
	processed = 0		# number of files processed
	for file in filenames:
		# bypass if not the correct file extension
		if not file.endswith(g_fileext): 
			continue
		# process the file
		filepath = os.path.join(dirpath, file)
		outpath = g_outdir + os.path.join(dirpath[len(g_indir):], file)
		move_or_copy_file(filepath, outpath, file)
		processed += 1
	return processed

# -----------------------------------------------------------------------------
		
def remove_hidden_directories(dirnames):
	"""Remove hidden directories in the dirnames list (don't process)."""
	for dir in dirnames:
		if dir[0] == '.': 
			dirnames.remove(dir)

# -----------------------------------------------------------------------------

def validate_global_variables():
	"""Validate the value of global variables."""
	if not os.path.isdir(g_indir):
		print "Input directory does not exist!"
		print
		return False
	if g_indir == g_outdir:
		print "Input and output directories are the same!"
		print
		return False
	if len(g_fileext) == 0:
		print "Invalid \"g_fileext\" value!"
		print
		return False
	if (g_copyfiles != "COPY"
	and g_copyfiles != "MOVE"):
		print "Invalid \"g_copyfiles\" value!"
		print
		return False
	return True

# -----------------------------------------------------------------------------

# make the module executable
if __name__ == "__main__":
	process_directories()
	# pause if script is not run from SciTE (argv[1] = 'scite')
	if (os.name == "nt"
	and len(sys.argv) == 1):
		print
		os.system("pause")
		# raw_input("\nPress Enter to continue . . .")

# -----------------------------------------------------------------------------



# Copyright (C) 2016 EDF
# All Rights Reserved
# This code is published under the GNU Lesser General Public License (GNU LGPL)
import numpy as np


# Defines a simple gas storage for optimization and simulation
# No constraint on the storage at the end of optimization period (so the storage will be empty)
class OptimizeGasStorageSwitchingCost :
    
    # Constructor
    # p_maxLevel          size of the storage
    # p_injectionRate     injection rate per time step
    # p_withdrawalRate    withdrawal rate between two time steps
    # p_injectionCost     injection cost
    # p_withdrawalCost    withdrawal cost
    def __init__(self, p_injectionRate, p_withdrawalRate, p_injectionCost, p_withdrawalCost, p_switchCost,  p_regime) :
        
        self.m_injectionRate = p_injectionRate
        self.m_withdrawalRate = p_withdrawalRate
        self.m_injectionCost = p_injectionCost
        self.m_withdrawalCost = p_withdrawalCost
        self.m_switchCost = p_switchCost
        self.m_regime = p_regime
        
    # define the diffusion cone for parallelism
    # p_regionByProcessor         region (min max) treated by the processor for the different regimes treated
    # returns in each dimension the min max values in the stock that can be reached from the grid p_gridByProcessor for each regime
    def getCone(self, p_regionByProcessor) :
        
        extrGrid = np.zeros(1)
        extrGrid[0][0] = p_regionByProcessor[0][0] - self.m_withdrawalRate
        extrGrid[0][1] = p_regionByProcessor[0][1] + self.m_injectionRate
        
        return extrGrid
    
    # get number of regimes
    def getNbRegime(self) :
  
        if isinstance(self.m_regime, int):
            return self.m_regime
        
        else:
            return self.m_regime.get(self.m_simulator.getCurrentStep())
    
    def getNbRegimeReached(self):
        
        if isinstance(self.m_regime, int):
            return self.m_regime
        
        else:
            return self.m_regime.get(self.m_simulator.getCurrentStep() + self.m_simulator.getStep())
        
    def getNbControl(self):
        
        return self.getNbRegime()
    
    # defines a step in optimization
    # p_grid      grid at arrival step after command
    # p_stock     coordinate of the stock point to treat
    # p_condEsp   conditional expectation operator
    # p_phiIn     for each regime  gives the solution calculated at the previous step ( next time step by Dynamic Programming resolution)
    # for each regimes (column) gives the solution for each particle (row)
    def stepOptimize(self, p_grid, p_stock, p_condEsp, p_phiIn) :
        
        # number of regime  allowed at the beginning of the time step
        nbReg = self.getNbRegime()
        actuStep= self.m_simulator.getActuStep()
        # number of regimes reached to test
        nbRegReached = self.getNbRegimeReached()
        nbSimul = self.m_simulator.getNbSimul()
        solutionAndControl = list(range(2))
        solutionAndControl[0] = np.zeros((nbSimul, nbReg))
        solutionAndControl[1] = np.zeros((nbSimul, nbReg))
        # Spot price
        spotPrice = self.m_simulator.fromParticlesToSpot(self.m_simulator.getParticles())
        
        # size of the stock
        maxStorage = p_grid.getExtremeValues()[0][1]
        # injection
        injectionMax = min(maxStorage - p_stock[0], self.m_injectionRate)
        # level min of the stock
        minStorage = p_grid.getExtremeValues()[0][0]
        withdrawalMax = min(p_stock[0] - minStorage, self.m_withdrawalRate)
        
        if (injectionMax < (0. - 1e3 * 2.220446049250313e-16)) & (withdrawalMax < (0. - 1e3 * 2.220446049250313e-16)):
            # not an admissible point
            solutionAndControl[0].setConstant(-1.e30)
            solutionAndControl[1].setConstant(0.)
            
            return solutionAndControl
        
        # Suppose that non injection and no withdrawal
        if p_grid.isInside(p_stock):
            # create interpolator at current stock point
            interpolatorCurrentStock = p_grid.createInterpolator(p_stock)
            # cash flow at current stock and previous step
            cashSameStock = interpolatorCurrentStock.applyVec(p_phiIn[0])
            # conditional expectation at current stock point
            condExpSameStock = actuStep * p_condEsp[0].getAllSimulations(interpolatorCurrentStock).squeeze()
            
        cashInjectionStock = []
        
        if (0. < (injectionMax - 1e3 * 2.220446049250313e-16)) & (nbRegReached >= 2):
            injectionStock = p_stock + injectionMax
            # interpolator for stock level if injection
            interpolatorInjectionStock = p_grid.createInterpolator(injectionStock)
            # cash flow  at previous step at injection level
            cashInjectionStock = interpolatorInjectionStock.applyVec(p_phiIn[1]).squeeze()
            # conditional expectation at injection stock level
            condExpInjectionStock = actuStep * p_condEsp[0].getAllSimulations(interpolatorInjectionStock).squeeze()
            
        cashWithdrawalStock = []
        
        # withdrawal
        if (0. < (withdrawalMax - 1e3 * 2.220446049250313e-16)) & (nbRegReached == 3): 
            withdrawalStock = p_stock - withdrawalMax
            # interpolator for stock level if withdrawal
            interpolatorWithdrawalStock = p_grid.createInterpolator(withdrawalStock)
            # cash flow  at previous step at injection level
            cashWithdrawalStock = interpolatorWithdrawalStock.applyVec(p_phiIn[2]).squeeze()
            # conditional expectation at withdrawal stock level
            condExpWithdrawalStock = actuStep * p_condEsp[0].getAllSimulations(interpolatorWithdrawalStock).squeeze()
            
        gainInjection = np.zeros((len(spotPrice), 3))
        gainWithdrawal = np.zeros((len(spotPrice), 3))
        gainSameStock = np.zeros((len(spotPrice), 3))
        # suppose that current regime is 0 (Do Nothing)
        gainInjection[:,0] =  - injectionMax * (spotPrice + self.m_injectionCost) - self.m_switchCost
        gainWithdrawal[:,0] =  withdrawalMax * (spotPrice - self.m_withdrawalCost) - self.m_switchCost
        # Regime 1) : injection
        gainInjection[:,1] =  - injectionMax * (spotPrice + self.m_injectionCost)
        gainWithdrawal[:,1] =  withdrawalMax * (spotPrice - self.m_withdrawalCost) - self.m_switchCost
        gainSameStock[:,1] = np.zeros(len(gainSameStock[:,1])) - self.m_switchCost
        # Regime 2) : withdrawal
        gainInjection[:,2] =  - injectionMax * (spotPrice + self.m_injectionCost) - self.m_switchCost
        gainWithdrawal[:,2] =  withdrawalMax * (spotPrice - self.m_withdrawalCost)
        gainSameStock[:,2] = np.zeros(len(gainSameStock[:,2])) - self.m_switchCost
        
        # now arbitrage in each regime
        for iReg in range(nbReg):
            
            if ((len(cashInjectionStock) > 0) & (len(cashWithdrawalStock) > 0)):
                
                if nbRegReached == 3:
                    # do the arbitrage
                    solutionAndControl[0][:,iReg] = gainSameStock[:,iReg] + np.multiply(actuStep, cashSameStock).transpose()
                    solutionAndControl[1][:,iReg] = 0.
                    espCondMax = condExpSameStock
                    espCondInjection = gainInjection[:,iReg] + condExpInjectionStock
                    
                    solutionAndControl[0][:,iReg] = np.where(espCondInjection > espCondMax, gainInjection[:,iReg] + actuStep * cashInjectionStock, solutionAndControl[0][:,iReg])
                    solutionAndControl[1][:,iReg] = np.where(espCondInjection > espCondMax, injectionMax, solutionAndControl[1][:,iReg])
                    espCondMax = np.where(espCondInjection > espCondMax, espCondInjection, espCondMax)
                    
                    espCondWithdrawal = gainWithdrawal[:,iReg] + condExpWithdrawalStock
                    
                    solutionAndControl[0][:,iReg] = np.where(espCondWithdrawal > espCondMax, gainWithdrawal[:,iReg] + actuStep * cashWithdrawalStock, solutionAndControl[0][:,iReg])
                    solutionAndControl[1][:,iReg] = np.where(espCondWithdrawal > espCondMax, - withdrawalMax, solutionAndControl[1][:,iReg])
                                    
                elif nbRegReached == 2:
                    # do the arbitrage
                    solutionAndControl[0][:,iReg] = gainSameStock[:,iReg] + np.multiply(actuStep, cashSameStock).transpose()
                    solutionAndControl[1][:,iReg] = 0.
                    espCondMax = condExpSameStock
                    espCondInjection = gainInjection[:,iReg] + condExpInjectionStock
                    
                    solutionAndControl[0][:,iReg] = np.where(espCondInjection > espCondMax, gainInjection[:,iReg] + actuStep * cashInjectionStock, solutionAndControl[0][:,iReg])
                    solutionAndControl[1][:,iReg] = np.where(espCondInjection > espCondMax, injectionMax, solutionAndControl[1][:,iReg])
                        
                else:
                    solutionAndControl[0][:,iReg] = gainSameStock[:,iReg] + np.multiply(actuStep, cashSameStock).transpose()
                    solutionAndControl[1][:,iReg] = 0.
                    
            elif len(cashWithdrawalStock) > 0:
                
                if len(cashSameStock) > 0:
                    
                    if nbRegReached == 3:
                        # do the arbitrage
                        solutionAndControl[0][:,iReg] = np.multiply(actuStep, cashSameStock).transpose()
                        solutionAndControl[1][:,iReg] = 0.
                        espCondMax = condExpSameStock - self.m_switchCost
                        espCondWithdrawal = gainWithdrawal[:,iReg] + condExpWithdrawalStock
                        
                        solutionAndControl[0][:,iReg] = np.where(espCondWithdrawal > espCondMax, gainWithdrawal[:,iReg] + actuStep * cashWithdrawalStock, solutionAndControl[0][:,iReg])
                        solutionAndControl[1][:,iReg] = np.where(espCondWithdrawal > espCondMax, - withdrawalMax, solutionAndControl[1][:,iReg])
                        
                    else:
                        solutionAndControl[0][:,iReg] = np.multiply(actuStep, cashSameStock).transpose()
                        solutionAndControl[1][:,iReg] = 0.
                        
                else:
                    
                    if nbRegReached == 3:
                        solutionAndControl[0][:,iReg] = gainWithdrawal[:,iReg] + np.multiply(actuStep, cashWithdrawalStock).transpose()
                        solutionAndControl[1][:,iReg] = - withdrawalMax
                    
                    else:
                        solutionAndControl[0][:,iReg].setConstant(-1.e30)
                        solutionAndControl[1][:,iReg] = 0.
                        
            elif len(cashInjectionStock) > 0:
                
                if len(cashSameStock) > 0:
                    
                    if nbRegReached >= 2:
                        # do the arbitrage
                        solutionAndControl[0][:,iReg] = np.multiply(actuStep, cashSameStock).transpose()
                        solutionAndControl[1][:,iReg] = 0.
                        espCondMax = condExpSameStock
                        espCondInjection = gainInjection[:,iReg] + condExpInjectionStock
                        
                        solutionAndControl[0][:,iReg] = np.where(espCondInjection > espCondMax, gainInjection[:,iReg] + actuStep * cashInjectionStock, solutionAndControl[0][:,iReg])
                        solutionAndControl[1][:,iReg] = np.where(espCondInjection > espCondMax, injectionMax, solutionAndControl[1][:,iReg])
                        
                    else:
                        solutionAndControl[0][:,iReg] = np.multiply(actuStep, cashSameStock).transpose()
                        solutionAndControl[1][:,iReg] = 0.
                        
                else:
                    
                    if nbRegReached >= 2:
                        solutionAndControl[0][:,iReg] = gainInjection[:,iReg] + actuStep * cashInjectionStock
                        solutionAndControl[1][:,iReg] = injectionMax
                        
                    else:
                        solutionAndControl[0][:,iReg].setConstant(-1.e30)
                        solutionAndControl[1][:,iReg] = 0.
                        
            else:
                
                # only same level
                if len(cashSameStock) > 0:
                    solutionAndControl[0][:,iReg] = np.multiply(actuStep, cashSameStock).transpose()
                    solutionAndControl[1][:,iReg] = 0.
                
                else:
                    solutionAndControl[0][:,iReg].setConstant(-1.e30)
                    solutionAndControl[1][:,iReg] = 0.
                    
        return solutionAndControl
    
    # defines a step in simulation
    # Notice that this implementation is not optimal. In fact no interpolation is necessary for this asset.
    # This implementation is for test and example purpose
    # p_grid          grid at arrival step after command
    # p_continuation  defines the continuation operator for each regime
    # p_state         defines the state value (modified)
    # p_phiInOut      defines the value function (modified)
    def stepSimulate(self, p_grid, p_continuation, p_state, p_phiInOut) :

        # actualization
        actuStep = self.m_simulator.getActuStep()
        actu = self.m_simulator.getActu()
        # optimal stock attained
        ptStockMax = np.zeros(p_state.getPtStock())
        # spot price
        spotPrice = self.m_simulator.fromOneParticleToSpot(p_state.getStochasticRealization())
        # if do nothing
        continuationDoNothing = actuStep * p_continuation[0].getValue(p_state.getPtStock(), p_state.getStochasticRealization())
        espCondMax = continuationDoNothing
        # gain to add at current point
        phiAdd = 0
        # size of the stock
        maxStorage = p_grid.getExtremeValues()[0][1]
        # if injection
        injectionMax = min(maxStorage - p_state.getPtStock()[0], self.m_injectionRate)
        gainInjection = -injectionMax * (spotPrice + self.m_injectionCost)
        continuationInjection = actuStep * p_continuation[1].getValue(p_state.getPtStock() + injectionMax, p_state.getStochasticRealization())
        espCondInjection = gainInjection + continuationInjection
        minStorage = p_grid.getExtremeValues()[0][0]
        withdrawalMax = min(p_state.getPtStock()[0] - minStorage, self.m_withdrawalRate)
        gainWithdrawal =  withdrawalMax * (spotPrice - self.m_withdrawalCost)
        continuationWithdrawal = self. m_actu * p_continuation[2].getValue(p_state.getPtStock() - withdrawalMax, p_state.getStochasticRealization())
        espCondWithdrawal = gainWithdrawal + continuationWithdrawal
        
        newRegime = 0
        
        if (p_state.getRegime() == 0):
            espCondInjection -= self.m_switchCost
            espCondWithdrawal -= self.m_switchCost
            
            if (espCondInjection > espCondMax):
                espCondMax = espCondInjection
                phiAdd = gainInjection - self.m_switchCost
                ptStockMax[0] += injectionMax
                newRegime = 1
            
            if (espCondWithdrawal > espCondMax):
                phiAdd = gainWithdrawal - self.m_switchCost
                ptStockMax[0] -= withdrawalMax
                newRegime = 2
            
        elif (p_state.getRegime() == 1):
            espCondMax -= self.m_switchCost
            espCondWithdrawal -= self.m_switchCost
            
            if (espCondInjection > espCondMax):
                espCondMax = espCondInjection
                phiAdd = gainInjection
                ptStockMax[0] += injectionMax
                newRegime = 1
            
            else:
                phiAdd = -self.m_switchCost
                newRegime = 0
            
            if (espCondWithdrawal > espCondMax):
                phiAdd = gainWithdrawal - self.m_switchCost
                ptStockMax[0] -= withdrawalMax
                newRegime = 2
            
        else: # regime withdrawal
            espCondMax -= self.m_switchCost
            espCondInjection -= self.m_switchCost
            
            if (espCondInjection > espCondMax):
                espCondMax = espCondInjection
                phiAdd = gainInjection - self.m_switchCost
                ptStockMax[0] += injectionMax
                newRegime = 1
            
            else:
                phiAdd = -self.m_switchCost
                newRegime = 0
            
            if (espCondWithdrawal > espCondMax):
                phiAdd = gainWithdrawal
                ptStockMax[0] -= withdrawalMax
                newRegime = 2
            
        # for return
        p_state.setPtStock(ptStockMax)
        p_state.setRegime(newRegime)
        p_phiInOut += phiAdd * actu
        
    # Defines a step in simulation using interpolation in controls
    # p_grid          grid at arrival step after command
    # p_control       defines the controls
    # p_state         defines the state value (modified)
    # p_phiInOut      defines the value function (modified): size number of functions to follow
    def stepSimulateControl(self, p_grid, p_control, p_state, p_phiInOut):
        
        # actualization
        actu = self.m_simulator.getActu()
        ptStock = p_state.getPtStock()
        iReg = p_state.getRegime()
        # spot price
        spotPrice = self.m_simulator.fromOneParticleToSpot(p_state.getStochasticRealization())
        # optimal control
        control = p_control[iReg].getValue(p_state.getPtStock(), p_state.getStochasticRealization())
        maxStorage = p_grid.getExtremeValues()[0][1]
        minStorage = p_grid.getExtremeValues()[0][0]
        control = max(min(maxStorage - ptStock[0], control), minStorage - ptStock[0])
        
        if control > 0:
            # already injection
            p_phiInOut[0] -= control * (spotPrice + self.m_injectionCost) * actu
            p_state.setRegime(1)
            
            if iReg != 1:
                p_phiInOut[0] -= self.m_switchCost
                p_state.setRegime(1)
                
        elif control < 0:
            p_phiInOut[0] -= control * (spotPrice - self.m_withdrawalCost) * actu
            
            if iReg != 2:
                p_phiInOut[0] -= self.m_switchCost
                p_state.setRegime(2)
                
        else:
            
            # do nothing
            if iReg != 0:
                p_phiInOut[0] -= self.m_switchCost
                p_state.setRegime(0)
                
        ptStock[0] += control
        p_state.setPtStock(ptStock)
        

    # store the simulator
    def setSimulator(self, p_simulator):
        
        self.m_simulator = p_simulator
        
    # get the simulator back
    def getSimulator(self):
        
        return self.m_simulator
    
    # get size of the  function to follow in simulation
    def getSimuFuncSize(self):
        
        return 1



# Copyright (C) 2016 EDF
# All Rights Reserved
# This code is published under the GNU Lesser General Public License (GNU LGPL)

# Permits to define a state containing
#   - a stock positipon
#   - a realisation of the non controled stochastic state


# State class dealing with regime, stock and uncertainty
class StateWithStocks :
    
    # constructor
    def __init__(self, p_regime, p_ptStock, p_stochasticRealisation) :
        
        self.m_ptStock = p_ptStock
        self.m_stochasticRealisation = p_stochasticRealisation
        self.m_regime = p_regime
        
    # accessor

    def getPtStock(self) :
        
        return self.m_ptStock
    
    def getStochasticRealization(self) :
        
        return self.m_stochasticRealisation
    
    def getRegime(self) :
        
        return self.m_regime
    
    def setPtStock(self, p_ptStock) :
        
        self.m_ptStock = p_ptStock
        
    def setStochasticRealization(self, p_stochasticRealisation) :
        
        self.m_stochasticRealisation = p_stochasticRealisation
        
    def setRegime(self, p_regime) :
        
        self.m_regime = p_regime



# Copyright (C) 2016 EDF
# All Rights Reserved
# This code is published under the GNU Lesser General Public License (GNU LGPL)
import numpy as np
import math as maths

# american option by Longstaff-Schwartz
# p_sim        Monte Carlo simulator
# p_payOff     Option pay off
# p_regressor  regressor object
def resolution(p_simulator, p_payOff, p_regressor) :
    
    step = p_simulator.getStep()
    # asset simulated under the neutral risk probability : get the trend of first asset to get interest rate
    expRate = np.exp(-step * p_simulator.getMu()[0])
    # Terminal
    particle = p_simulator.getParticles()
    Cash = p_payOff.operator(particle)
    
    for iStep in range(0, p_simulator.getNbStep()):
        asset = p_simulator.stepBackwardAndGetParticles()
        payOffLoc = p_payOff.operator(asset)
        isLastStep = False        
        if iStep == p_simulator.getNbStep() - 1 :
            isLastStep = True
    
        p_regressor.updateSimulations(isLastStep, asset)
        # conditional expectation
        condEspec = p_regressor.getAllSimulations(Cash).squeeze() * expRate
        # arbitrage
        Cash = np.where(condEspec < payOffLoc, payOffLoc, Cash * expRate) 
       
    return maths.fsum(Cash) / len(Cash)



# Copyright (C) 2016 EDF
# All Rights Reserved
# This code is published under the GNU Lesser General Public License (GNU LGPL)
import numpy as np
import unittest
import random
import math
import StOptGrids

# function used
def funcToInterpolate( x):
    return math.log(1. + x.sum())

# unit test for sparse grids
############################

class testGrids(unittest.TestCase):


    # test sparse grids  with boundaries
    def testSparseGridsBounds(self):
        # low values
        lowValues =np.array([1.,2.,3.])
        # size of the domain
        sizeDomValues = np.array([3.,4.,3.])
        # anisotropic weights
        weights =  np.array([1.,1.,1.])
        # level of the sparse grid
        level =3
        # create the sparse grid with linear interpolator
        sparseGridLin = StOptGrids.SparseSpaceGridBound(lowValues,sizeDomValues, level, weights,1)
        iterGrid = sparseGridLin.getGridIterator()
         # array to store
        data = np.empty(sparseGridLin.getNbPoints())
        # iterates on point
        while( iterGrid.isValid()):
          data[iterGrid.getCount()] = funcToInterpolate(iterGrid.getCoordinate())
          iterGrid.next()
        # Hierarchize the data
        hierarData = sparseGridLin.toHierarchize(data)
        # get back an interpolator
        ptInterp = np.array([2.3,3.2,5.9],dtype=float)
        interpol = sparseGridLin.createInterpolator(ptInterp)
        # calculate interpolated value
        interpValue = interpol.apply(hierarData)
        print(("Interpolated value sparse linear" , interpValue))
         # create the sparse grid with quadratic interpolator
        sparseGridQuad = StOptGrids.SparseSpaceGridBound(lowValues,sizeDomValues, level, weights,2)
        # Hierarchize the data
        hierarData = sparseGridQuad.toHierarchize(data)
        # get back an interpolator
        ptInterp = np.array([2.3,3.2,5.9],dtype=float)
        interpol = sparseGridQuad.createInterpolator(ptInterp)
        # calculate interpolated value
        interpValue = interpol.apply(hierarData)
        print(("Interpolated value sparse quadratic " , interpValue))
        # now refine
        precision = 1e-6
        print(("Size of hierarchical array " , len(hierarData)))
        valueAndHierar = sparseGridQuad.refine(precision,funcToInterpolate,data,hierarData)
        print(("Size of hierarchical array after refinement " , len(valueAndHierar[0])))
        # calculate interpolated value
        interpol1 = sparseGridQuad.createInterpolator(ptInterp)
        interpValue = interpol1.apply(valueAndHierar[1])
        print(("Interpolated value sparse quadratic after refinement " , interpValue))
        # coarsen the grid
        precision = 1e-4
        valueAndHierarCoarsen = sparseGridQuad.coarsen(precision,valueAndHierar[0],valueAndHierar[1])
        print(("Size of hierarchical array after coarsening " , len(valueAndHierarCoarsen[0])))
        # calculate interpolated value
        interpol2 = sparseGridQuad.createInterpolator(ptInterp)
        interpValue = interpol2.apply(valueAndHierarCoarsen[1])
        print(("Interpolated value sparse quadratic after refinement " , interpValue))
         
 
    # test sparse grids eliminating boundaries
    def testSparseGridsNoBounds(self):
        # low values
        lowValues =np.array([1.,2.,3.],dtype=float)
        # size of the domain
        sizeDomValues = np.array([3.,4.,3.],dtype=float)
        # anisotropic weights
        weights =  np.array([1.,1.,1.])
        # level of the sparse grid
        level =3
        # create the sparse grid with linear interpolator
        sparseGridLin = StOptGrids.SparseSpaceGridNoBound(lowValues,sizeDomValues, level, weights,1)
        iterGrid = sparseGridLin.getGridIterator()
         # array to store
        data = np.empty(sparseGridLin.getNbPoints())
        # iterates on point
        while( iterGrid.isValid()):
          data[iterGrid.getCount()] = funcToInterpolate(iterGrid.getCoordinate())
          iterGrid.next()
        # Hierarchize the data
        hierarData = sparseGridLin.toHierarchize(data)
        # get back an interpolator
        ptInterp = np.array([2.3,3.2,5.9],dtype=float)
        interpol = sparseGridLin.createInterpolator(ptInterp)
        # calculate interpolated value
        interpValue = interpol.apply(hierarData)
        print(("Interpolated value sparse linear" , interpValue))
         # create the sparse grid with quadratic interpolator
        sparseGridQuad = StOptGrids.SparseSpaceGridNoBound(lowValues,sizeDomValues, level, weights,2)
        # Hierarchize the data
        hierarData = sparseGridQuad.toHierarchize(data)
        # get back an interpolator
        ptInterp = np.array([2.3,3.2,5.9],dtype=float)
        interpol = sparseGridQuad.createInterpolator(ptInterp)
        # calculate interpolated value
        interpValue = interpol.apply(hierarData)
        print(("Interpolated value sparse quadratic " , interpValue))
        # test grids function
        iDim = sparseGridQuad.getDimension()
        pt = sparseGridQuad.getExtremeValues()
        # now refine
        precision = 1e-6
        print(("Size of hierarchical array " , len(hierarData)))
        valueAndHierar = sparseGridQuad.refine(precision,funcToInterpolate,data,hierarData)
        print(("Size of hierarchical array after refinement " , len(valueAndHierar[0])))
        # calculate interpolated value
        interpol1 = sparseGridQuad.createInterpolator(ptInterp)
        interpValue = interpol1.apply(valueAndHierar[1])
        print(("Interpolated value sparse quadratic after coarsening " , interpValue))
        # coarsen the grid
        precision = 1e-4
        valueAndHierarCoarsen = sparseGridQuad.coarsen(precision,valueAndHierar[0],valueAndHierar[1])
        print(("Size of hierarchical array after coarsening " , len(valueAndHierarCoarsen[0])))
        # calculate interpolated value
        interpol2 = sparseGridQuad.createInterpolator(ptInterp)
        interpValue = interpol2.apply(valueAndHierarCoarsen[1])
        print(("Interpolated value sparse quadratic after coarsening " , interpValue))

if __name__ == '__main__': 
    unittest.main()
